/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.common.graph;

import de.tudarmstadt.maki.simonstrator.api.Graphs;

public class GraphBuilder {

	private Graph graph;

	private GraphBuilder() {

		this.graph = Graphs.createGraph();
	}

	public static GraphBuilder create() {
		return new GraphBuilder();
	}

	public Graph done() {
		return this.graph;
	}

	public GraphBuilder addEdge(String sourceId, String targetId) {
		this.graph.addEdge(
				new DirectedEdge(INodeID.get(sourceId), INodeID.get(targetId)));
		return this;
	}

	public GraphBuilder addEdge(String sourceId, String targetId,
			String edgeId) {
		INodeID sourceNodeId = INodeID.get(sourceId);
		INodeID targetNodeId = INodeID.get(targetId);
		if (!this.graph.containsNode(sourceNodeId)) {
			this.graph.createAndAddNode(sourceNodeId);
		}
		if (!this.graph.containsNode(targetNodeId)) {
			this.graph.createAndAddNode(targetNodeId);
		}
		this.graph.addEdge(new DirectedEdge(sourceNodeId, targetNodeId,
				EdgeID.get(targetId)));
		return this;
	}

	public GraphBuilder addEdge(String sourceId, String targetId, String edgeId,
			double weight) {
		INodeID sourceNodeId = INodeID.get(sourceId);
		INodeID targetNodeId = INodeID.get(targetId);
		if (!this.graph.containsNode(sourceNodeId)) {
			this.graph.createAndAddNode(sourceNodeId);
		}
		if (!this.graph.containsNode(targetNodeId)) {
			this.graph.createAndAddNode(targetNodeId);
		}
		this.graph.addEdge(Graphs.createDirectedWeightedEdge(
				INodeID.get(sourceId), INodeID.get(targetId), weight));
		return this;
	}
}
